// (C) Copyright 2004, David M. Blei (blei [at] cs [dot] cmu [dot] edu)

// This file is part of LDA-C.

// LDA-C is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.

// LDA-C is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA

#include "fdlda-inference.h"

/*
 * variational inference
 *
 */

double lda_inference2(document* doc, lda_model* model, double* var_gamma, double* phi)
{
    double converged = 1;
    double phisum = 0, likelihood = 0;
    double likelihood_old = 0, oldphi[model->num_topics];
    int k, n, var_iter;
    double digamma_gam[model->num_topics];
    double log_beta[model->num_topics];

 
    // compute posterior dirichlet

    //printf("inf1\n");
    for (k = 0; k < model->num_topics; k++)
    {
        var_gamma[k] = model->alpha + (doc->total/((double) model->num_topics));
        digamma_gam[k] = digamma(var_gamma[k]);
        for (n = 0; n < doc->length; n++)
            phi[k] = 1.0/model->num_topics;
    }
    var_iter = 0;


   for (k = 0; k < model->num_topics; k++){
       log_beta[k]=0;
   }

   for (k = 0; k < model->num_topics; k++){
     for (n = 0; n < doc->length; n++){
        log_beta[k]+=model->log_prob_w[k][doc->words[n]];
     }
   }

   for (k = 0; k < model->num_topics; k++){
       log_beta[k]=(1.0/(double)doc->total)*log_beta[k];
   }


    //printf("inf2");
    while ((converged > VAR_CONVERGED) &&
           ((var_iter < VAR_MAX_ITER) || (VAR_MAX_ITER == -1)))
    {
	var_iter++;

            phisum = 0;
            for (k = 0; k < model->num_topics; k++)
            {
                oldphi[k] = phi[k];
                phi[k] =
                    digamma_gam[k] +
                    log_beta[k];

                if (k > 0)
                    phisum = log_sum(phisum, phi[k]);
                else
                    phisum = phi[k]; // note, phi is in log space
            }

            for (k = 0; k < model->num_topics; k++)
            {
                phi[k] = exp(phi[k] - phisum);
                var_gamma[k] =
                    var_gamma[k] + (double)doc->total*(phi[k] - oldphi[k]);
                // !!! a lot of extra digamma's here because of how we're computing it
                // !!! but its more automatically updated too.
                digamma_gam[k] = digamma(var_gamma[k]);
            }
        

        likelihood = compute_likelihood2(doc, model, phi, var_gamma);
        //printf("LOGLIKE: %f DOC-COUNT:%d, TOTAL:%d\n",likelihood, doc->length,doc->xz,doc->total);
        assert(!isnan(likelihood));
        converged = (likelihood_old - likelihood) / likelihood_old;
        likelihood_old = likelihood;
        if(var_iter%100==0){
            //printf("ITERATIONS:%d\n",var_iter);
        }
        // printf("[LDA INF] %8.5f %1.3e\n", likelihood, converged);
    }
    return(likelihood);
}


double lda_inference(document* doc, lda_model* model, double* var_gamma, double* phi)
{
    double converged = 1;
    double phisum = 0, likelihood = 0;
    double likelihood_old = 0, oldphi[model->num_topics];
    int k, n, var_iter;
    double digamma_gam[model->num_topics];
    double log_beta[model->num_topics];
    double last_term[model->num_topics];
    //double old_phi[model->num_topics];
    double tmpsum=0;

    // compute posterior dirichlet

    for (k = 0; k < model->num_topics; k++)
    {
        var_gamma[k] = model->alpha + (doc->total/((double) model->num_topics));
        digamma_gam[k] = digamma(var_gamma[k]);
        phi[k] = 1.0/model->num_topics;
        //init phi sum by topic 
        doc->phi_sum[k]=0; 
    }


   doc->xz=0.0;
   //go through all topics
   for(k=0; k<model->num_topics;k++){
      //printf("XZ: %f\n",doc->xz);
       doc->xz+=phi[k]*exp(model->nu[k]);
      //printf("PROB:%f, EXPETA:%f XZ:%f\n",doc->counts[n]*phi[n][k],exp(model->nu[k]),doc->xz);
   }
   
   doc->xz=doc->xz+1;

   var_iter = 0;

   
   for (k = 0; k < model->num_topics; k++){
       log_beta[k]=0;
   }

   for (k = 0; k < model->num_topics; k++){
     for (n = 0; n < doc->length; n++){
        log_beta[k]+=doc->counts[n]*model->log_prob_w[k][doc->words[n]];
     }
   }
  
   for (k = 0; k < model->num_topics; k++){
       log_beta[k]=(1.0/(double)doc->total)*log_beta[k];
       last_term[k]=(1.0/(double)doc->total)*(model->nu[k]*doc->label-1/doc->xz*exp(model->nu[k]));
       if(isnan(last_term[k])){
          printf("TOTAL:%d NU:%f XZ:%f label:%d\n",doc->total,model->nu[k],doc->xz,doc->label);
       }
   }


    while ((converged > VAR_CONVERGED) &&
           ((var_iter < VAR_MAX_ITER) || (VAR_MAX_ITER == -1)))
    {
	var_iter++;  

       
            phisum = 0;

            for (k = 0; k < model->num_topics; k++)
            {
                oldphi[k] = phi[k];
                phi[k] =
                    digamma_gam[k] +
                    log_beta[k]+
                    last_term[k];

                if (k > 0)
                    phisum = log_sum(phisum, phi[k]);
                else
                    phisum = phi[k]; // note, phi is in log space
            }

            
     
 
            for (k = 0; k < model->num_topics; k++)
            {
               
                phi[k] = exp(phi[k] - phisum);
                var_gamma[k] =
                    var_gamma[k] + (double)doc->total*(phi[k] - oldphi[k]);
                // !!! a lot of extra digamma's here because of how we're computing it
                // !!! but its more automatically updated too.
                digamma_gam[k] = digamma(var_gamma[k]);
             
                //printf("phi[%d][%d]=%f\n",n,k,phi[n][k]);
            }
             
         for (k = 0; k < model->num_topics; k++)
         {
            tmpsum+=phi[k];
         }        
       

        likelihood = compute_likelihood(doc, model, phi, var_gamma);
        if(isnan(likelihood) || tmpsum<0){
            for (k = 0; k < model->num_topics; k++)
            {
                 printf("phi[%d]=%f,dig=%f,log_beta=%f,last_term=%f\n",k,phi[k],digamma_gam[k],log_beta[k],last_term[k]);
            }
        }
 
        //printf("LOGLIKE: %f DOC-COUNT:%d, XZ:%f, TOTAL:%d\n",likelihood, doc->length,doc->xz,doc->total);
        assert(!isnan(likelihood));
        converged = (likelihood_old - likelihood) / likelihood_old;
        //printf("LOGLIKE: %f ERROR:%f\n",likelihood,converged);
        likelihood_old = likelihood;
       
        if(var_iter%100==0){
          // printf("ITERATIONS:%d\n",var_iter);
        }  
        //printf("[LDA INF] %8.5f %1.3e\n", likelihood, converged);
    }
    return(likelihood);
}


/*
 * compute likelihood bound
 *
 */

double
compute_likelihood(document* doc, lda_model* model, double* phi, double* var_gamma)
{
    double likelihood = 0, digsum = 0, var_gamma_sum = 0, dig[model->num_topics];
    int k, n;
  
  

   

    for (k = 0; k < model->num_topics; k++)
    {
	dig[k] = digamma(var_gamma[k]);
	var_gamma_sum += var_gamma[k];
    }
    digsum = digamma(var_gamma_sum);

    likelihood =
	lgamma(model->alpha * model -> num_topics)
	- model -> num_topics * lgamma(model->alpha)
	- (lgamma(var_gamma_sum));

    for (k = 0; k < model->num_topics; k++)
    {
	likelihood +=
	    (model->alpha - 1)*(dig[k] - digsum) + lgamma(var_gamma[k])
	    - (var_gamma[k] - 1)*(dig[k] - digsum);

	for (n = 0; n < doc->length; n++)
	{
            if (phi[k] > 0)
            {
                likelihood += doc->counts[n]*
                    (phi[k]*((dig[k] - digsum) - log(phi[k])
                                + doc->counts[n]*model->log_prob_w[k][doc->words[n]])
                        +(1.0/(double)doc->total)*(model->nu[k]*doc->label-1/doc->xz*exp(model->nu[k])));
            }
        }
    }
    likelihood +=(1-1/doc->xz-log(doc->xz));
    if(isnan(likelihood)){
       printf("TOTAL:%d XZ:%f\n",doc->total,doc->xz);
       for(k = 0; k < model->num_topics; k++){
           printf(" PHI[%d]: %f nu[%d]: %f\n",k,phi[k],k,model->nu[k]);
       }
    }
    return(likelihood);
}


/*
 * compute likelihood bound
 *
 */

double
compute_likelihood2(document* doc, lda_model* model, double* phi, double* var_gamma)
{
    double likelihood = 0, digsum = 0, var_gamma_sum = 0, dig[model->num_topics];
    int k, n;

 

    for (k = 0; k < model->num_topics; k++)
    {
	dig[k] = digamma(var_gamma[k]);
	var_gamma_sum += var_gamma[k];
    }
    digsum = digamma(var_gamma_sum);

    likelihood =
	lgamma(model->alpha * model -> num_topics)
	- model -> num_topics * lgamma(model->alpha)
	- (lgamma(var_gamma_sum));

    for (k = 0; k < model->num_topics; k++)
    {
	likelihood +=
	    (model->alpha - 1)*(dig[k] - digsum) + lgamma(var_gamma[k])
	    - (var_gamma[k] - 1)*(dig[k] - digsum);

	for (n = 0; n < doc->length; n++)
	{
            if (phi[k] > 0)
            {
                likelihood += doc->counts[n]*
                    (phi[k]*((dig[k] - digsum) - log(phi[k])
                                + doc->counts[n]*model->log_prob_w[k][doc->words[n]]));
            }
        }
    }
    return(likelihood);
}
